@@ -353,6 +353,7 @@ class LenovoFirmwareConfig(object):
                     reset = False
                     name = setting.find("mriName").text
                     help = setting.find("desc").text
+                    validexpression = None
                     onedata = setting.find('text_data')
                     if onedata is not None:
                         if onedata.get('password') == 'true':
@@ -366,11 +367,18 @@ class LenovoFirmwareConfig(object):
                     if onedata is not None:
                         if onedata.get('maxinstance') is not None:
                             forceinstance = True
+                        validexpression = onedata.get('pattern', None)
                         instances = list(onedata.iter('instance'))
                         if not instances:
                             protect = True  # not supported yet
                         else:
-                            current = [x.text for x in instances]
+                            instbynum = {}
+                            defidx = 1
+                            for x in instances:
+                                xid = int(x.get('ID', defidx))
+                                instbynum[xid] = x
+                                defidx += 1
+                            current = [instbynum[idx].text for idx in sorted(instbynum)]
                         default = onedata.get('default', None)
                         if default == '':
                             default = None
@@ -451,6 +459,7 @@ class LenovoFirmwareConfig(object):
                                     readonly_expression=readonly,
                                     hide_expression=hide,
                                     sortid=sortid,
+                                    validexpression=validexpression,
                                     alias=alias)
                                 sortid += 1
                                 instidx += 1
@@ -479,6 +488,7 @@ class LenovoFirmwareConfig(object):
                                 readonly_expression=readonly,
                                 hide_expression=hide,
                                 sortid=sortid,
+                                validexpression=validexpression,
                                 alias=alias)
                             sortid += 1
                         continue
@@ -503,6 +513,7 @@ class LenovoFirmwareConfig(object):
                                                readonly_expression=readonly,
                                                hide_expression=hide,
                                                sortid=sortid,
+                                               validexpression=validexpression,
                                                alias=alias)
                     sortid = sortid + 1
         for opt in options:
