@@ -346,7 +346,7 @@ class OEMHandler(generic.OEMHandler):
             return self.smmhandler.get_ntp_servers()
         if self.has_tsma:
             return self.tsmahandler.get_ntp_servers()
-        return None
+        return ()
 
     def set_ntp_enabled(self, enabled):
         if self.has_tsm or self.has_ami or self.has_asrock:
@@ -1191,7 +1191,7 @@ class OEMHandler(generic.OEMHandler):
         if self.has_xcc:
             return self.immhandler.get_description()
         if self.is_fpc:
-            return {'height': self._fpc_variant, 'slot': 0}
+            return {'height': self._fpc_variant & 0xf, 'slot': 0}
         return super(OEMHandler, self).get_description()
 
     def get_extended_bmc_configuration(self):
