@@ -16,9 +16,39 @@ from fnmatch import fnmatch
 import json
 import os
 import re
+import time
 
+import pyghmi.constants as const
 import pyghmi.exceptions as exc
 import pyghmi.media as media
+import pyghmi.util.webclient as webclient
+
+
+class SensorReading(object):
+    def __init__(self, healthinfo, sensor=None, value=None, units=None,
+                 unavailable=False):
+        if sensor:
+            self.name = sensor['name']
+        else:
+            self.name = healthinfo['Name']
+            self.health = _healthmap.get(healthinfo.get(
+                'Status', {}).get('Health', None), const.Health.Warning)
+            self.states = [healthinfo.get('Status', {}).get('Health',
+                                                            'Unknown')]
+            self.health = _healthmap[healthinfo['Status']['Health']]
+            self.states = [healthinfo['Status']['Health']]
+        self.value = value
+        self.state_ids = None
+        self.imprecision = None
+        self.units = units
+        self.unavailable = unavailable
+
+_healthmap = {
+    'Critical': const.Health.Critical,
+    'Unknown': const.Health.Warning,
+    'Warning': const.Health.Warning,
+    'OK': const.Health.Ok,
+}
 
 boot_devices_write = {
     'net': 'Pxe',
@@ -133,6 +163,74 @@ class OEMHandler(object):
         self._urlcache = cache
         self.webclient = webclient
 
+    def get_health(self, fishclient, verbose=True):
+        health = fishclient.sysinfo.get('Status', {})
+        health = health.get('HealthRollup', health.get('Health', 'Unknown'))
+        warnunknown = health == 'Unknown'
+        health = _healthmap[health]
+        summary = {'badreadings': [], 'health': health}
+        if health > 0 and verbose:
+            # now have to manually peruse all psus, fans, processors, ram,
+            # storage
+            procsumstatus = fishclient.sysinfo.get('ProcessorSummary', {}).get(
+                'Status', {})
+            procsumstatus = procsumstatus.get('HealthRollup',
+                                              procsumstatus.get('Health',
+                                                                None))
+            if procsumstatus != 'OK':
+                procfound = False
+                procurl = fishclient.sysinfo.get('Processors', {}).get('@odata.id',
+                                                                 None)
+                if procurl:
+                    for cpu in fishclient._do_web_request(procurl).get(
+                            'Members', []):
+                        cinfo = fishclient._do_web_request(cpu['@odata.id'])
+                        if cinfo.get('Status', {}).get(
+                                'State', None) == 'Absent':
+                            continue
+                        if cinfo.get('Status', {}).get(
+                                'Health', None) not in ('OK', None):
+                            procfound = True
+                            summary['badreadings'].append(SensorReading(cinfo))
+                if not procfound:
+                    procinfo = fishclient.sysinfo['ProcessorSummary']
+                    procinfo['Name'] = 'Processors'
+                    summary['badreadings'].append(SensorReading(procinfo))
+            memsumstatus = fishclient.sysinfo.get(
+                'MemorySummary', {}).get('Status', {})
+            memsumstatus = memsumstatus.get('HealthRollup',
+                                            memsumstatus.get('Health', None))
+            if memsumstatus != 'OK':
+                dimmfound = False
+                for mem in fishclient._do_web_request(
+                        fishclient.sysinfo['Memory']['@odata.id'])['Members']:
+                    dimminfo = fishclient._do_web_request(mem['@odata.id'])
+                    if dimminfo.get('Status', {}).get(
+                            'State', None) == 'Absent':
+                        continue
+                    if dimminfo.get('Status', {}).get(
+                            'Health', None) not in ('OK', None):
+                        summary['badreadings'].append(SensorReading(dimminfo))
+                        dimmfound = True
+                if not dimmfound:
+                    meminfo = fishclient.sysinfo['MemorySummary']
+                    meminfo['Name'] = 'Memory'
+                    summary['badreadings'].append(SensorReading(meminfo))
+            for adapter in fishclient.sysinfo['PCIeDevices']:
+                adpinfo = fishclient._do_web_request(adapter['@odata.id'])
+                if adpinfo['Status']['Health'] not in ('OK', None):
+                    summary['badreadings'].append(SensorReading(adpinfo))
+            for fun in fishclient.sysinfo['PCIeFunctions']:
+                funinfo = fishclient._do_web_request(fun['@odata.id'])
+                if funinfo['Status']['Health'] not in ('OK', None):
+                    summary['badreadings'].append(SensorReading(funinfo))
+        if warnunknown and not summary['badreadings']:
+            unkinf = SensorReading({'Name': 'BMC',
+                                    'Status': {'Health': 'Unknown'}})
+            unkinf.states = ['System does not provide health information']
+            summary['badreadings'].append(unkinf)
+        return summary
+
     def user_delete(self, uid):
         # Redfish doesn't do so well with Deleting users either...
         # Blanking the username seems to be the convention
@@ -431,7 +529,7 @@ class OEMHandler(object):
                 'UUID': self._varsysinfo.get('UUID', ''),
                 'Serial Number': self._varsysinfo.get('SerialNumber', ''),
                 'Manufacturer': self._varsysinfo.get('Manufacturer', ''),
-                'Product Name': self._varsysinfo.get('Model', ''),
+                'Product name': self._varsysinfo.get('Model', ''),
                 'Model': self._varsysinfo.get(
                     'SKU', self._varsysinfo.get('PartNumber', '')),
             }
@@ -446,7 +544,7 @@ class OEMHandler(object):
             'UUID': self._varsysinfo.get('UUID', ''),
             'Serial Number': self._varsysinfo.get('SerialNumber', ''),
             'Manufacturer': self._varsysinfo.get('Manufacturer', ''),
-            'Product Name': self._varsysinfo.get('Model', ''),
+            'Product name': self._varsysinfo.get('Model', ''),
             'Model': self._varsysinfo.get(
                 'SKU', self._varsysinfo.get('PartNumber', '')),
         }
@@ -681,8 +779,84 @@ class OEMHandler(object):
             'Remote media upload not supported on this platform')
 
     def update_firmware(self, filename, data=None, progress=None, bank=None):
-        raise exc.UnsupportedFunctionality(
-            'Firmware update not supported on this platform')
+        usd = self._do_web_request('/redfish/v1/UpdateService')
+        if usd.get('HttpPushUriTargetsBusy', False):
+            raise exc.TemporaryError('Cannot run multtiple updates to '
+                                        'same target concurrently')
+        try:
+            upurl = usd['HttpPushUri']
+        except KeyError:
+            raise exc.UnsupportedFunctionality('Redfish firmware update only supported for implementations with push update support')
+        if 'HttpPushUriTargetsBusy' in usd:
+            self._do_web_request(
+                '/redfish/v1/UpdateService',
+                {'HttpPushUriTargetsBusy': True}, method='PATCH')
+        try:
+            uploadthread = webclient.FileUploader(
+                self.webclient, upurl, filename, data, formwrap=False,
+                excepterror=False)
+            uploadthread.start()
+            wc = self.webclient
+            while uploadthread.isAlive():
+                uploadthread.join(3)
+                if progress:
+                    progress(
+                        {'phase': 'upload',
+                         'progress': 100 * wc.get_upload_progress()})
+            if (uploadthread.rspstatus >= 300
+                    or uploadthread.rspstatus < 200):
+                rsp = uploadthread.rsp
+                errmsg = ''
+                try:
+                    rsp = json.loads(rsp)
+                    errmsg = (
+                        rsp['error'][
+                            '@Message.ExtendedInfo'][0]['Message'])
+                except Exception:
+                    raise Exception(uploadthread.rsp)
+                raise Exception(errmsg)
+            rsp = json.loads(uploadthread.rsp)
+            monitorurl = rsp['@odata.id']
+            complete = False
+            phase = "apply"
+            statetype = 'TaskState'
+            # sometimes we get an empty pgress when transitioning from the apply phase to
+            # the validating phase; add a retry here so we don't exit the loop in this case
+            retry = 3
+            while not complete and retry > 0:
+                pgress = self._do_web_request(monitorurl, cache=False)
+                if not pgress:
+                    retry -= 1
+                    time.sleep(3)
+                    continue
+                for msg in pgress.get('Messages', []):
+                    if 'Verify failed' in msg.get('Message', ''):
+                        raise Exception(msg['Message'])
+                state = pgress[statetype]
+                if state in ('Cancelled', 'Exception', 'Interrupted',
+                             'Suspended'):
+                    raise Exception(
+                        json.dumps(json.dumps(pgress['Messages'])))
+                pct = float(pgress['PercentComplete'])
+                complete = state == 'Completed'
+                progress({'phase': phase, 'progress': pct})
+                if complete:
+                    msgs = pgress.get('Messages', [])
+                    if msgs and 'OperationTransitionedToJob' in msgs[0].get('MessageId', ''):
+                        monitorurl = pgress['Messages'][0]['MessageArgs'][0]
+                        phase = 'validating'
+                        statetype = 'JobState'
+                        complete = False
+                        time.sleep(3)
+                else:
+                    time.sleep(3)
+            return 'pending'
+        finally:
+            if 'HttpPushUriTargetsBusy' in usd:
+                self._do_web_request(
+                    '/redfish/v1/UpdateService',
+                    {'HttpPushUriTargetsBusy': False}, method='PATCH')
+
 
     def _do_bulk_requests(self, urls, cache=True):
         if self._gpool:
