@@ -24,7 +24,9 @@ import socket
 import time
 
 import six
+import zipfile
 
+import pyghmi.constants as pygconst
 import pyghmi.exceptions as pygexc
 import pyghmi.ipmi.private.util as util
 import pyghmi.ipmi.oem.lenovo.config as config
@@ -34,6 +36,21 @@ import pyghmi.storage as storage
 from pyghmi.util.parse import parse_time
 import pyghmi.util.webclient as webclient
 
+class SensorReading(object):
+    def __init__(self, healthinfo, sensor=None, value=None, units=None,
+                 unavailable=False):
+        if sensor:
+            self.name = sensor['name']
+        else:
+            self.name = healthinfo['name']
+            self.health = healthinfo['health']
+            self.states = healthinfo['states']
+            self.state_ids = healthinfo.get('state_ids', None)
+        self.value = value
+        self.imprecision = None
+        self.units = units
+        self.unavailable = unavailable
+
 numregex = re.compile('([0-9]+)')
 funtypes = {
     0: 'RAID Controller',
@@ -85,6 +102,22 @@ def natural_sort(iterable):
         return sorted(iterable)
 
 
+def str_to_size(sizestr):
+    if 'GB' in sizestr:
+        sizestr = sizestr.replace('GB', '')
+        sizestr = int(float(sizestr) * 1000)
+    elif 'GiB' in sizestr:
+        sizestr = sizestr.replace('GiB', '')
+        sizestr = int(float(sizestr) * 1024)
+    elif 'TB' in sizestr:
+        sizestr = sizestr.replace('TB', '')
+        sizestr = int(float(sizestr) * 1000 * 1000)
+    elif 'TiB' in sizestr:
+        sizestr = sizestr.replace('TiB', '')
+        sizestr = int(float(sizestr) * 1024 * 1024)
+    return sizestr
+
+
 class OEMHandler(generic.OEMHandler):
     logouturl = '/api/providers/logout'
     bmcname = 'XCC'
@@ -236,6 +269,11 @@ class OEMHandler(generic.OEMHandler):
                                 '({2})'.format(
                                     newvalue, key,
                                     ','.join(self.fwo[key]['possible'])))
+                elif self.fwo[key]['validexpression']:
+                    if not re.match(self.fwo[key]['validexpression'], newvalue):
+                        raise pygexc.InvalidParameterValue(
+                            '"{0}" does not match expression "{1}"'.format(
+                                newvalue, self.fwo[key]['validexpression']))
                 newnewvalues.append(newvalue)
             if len(newnewvalues) == 1:
                 self.fwo[key]['new_value'] = newnewvalues[0]
@@ -246,8 +284,11 @@ class OEMHandler(generic.OEMHandler):
         if bay != -1:
             raise pygexc.UnsupportedFunctionality(
                 'This is not an enclosure manager')
-        rsp = self.wc.grab_json_response_with_status(
+        wc = self.wc.dupe(timeout=5)
+        rsp = wc.grab_json_response_with_status(
             '/api/providers/virt_reseat', '{}')
+        if rsp[1] == 500 and rsp[0] == 'Target Unavailable':
+            return
         if rsp[1] != 200 or rsp[0].get('return', 1) != 0:
             raise pygexc.UnsupportedFunctionality(
                 'This platform does not support AC reseat.')
@@ -477,6 +518,60 @@ class OEMHandler(generic.OEMHandler):
                     self.wc.grab_json_response(
                         '/api/function', {'USB_AddMapping': newmapping})
 
+    def get_health(self, fishclient, verbose=True):
+        rsp = self._do_web_request('/api/providers/imm_active_events')
+        summary = {'badreadings': [], 'health': pygconst.Health.Ok}
+        fallbackdata = []
+        hmap = {
+            'I': pygconst.Health.Ok,
+            'E': pygconst.Health.Critical,
+            'W': pygconst.Health.Warning,
+        }
+        infoevents = False
+        existingevts = set([])
+        for item in rsp.get('items', ()):
+            # while usually the ipmi interrogation shall explain things,
+            # just in case there is a gap, make sure at least the
+            # health field is accurately updated
+            itemseverity = hmap.get(item.get('severity', 'E'),
+                                    pygconst.Health.Critical)
+            if itemseverity == pygconst.Health.Ok:
+                infoevents = True
+                continue
+            if (summary['health'] < itemseverity):
+                summary['health'] = itemseverity
+            if item['cmnid'] == 'FQXSPPW0104J':
+                # This event does not get modeled by the sensors
+                # add a made up sensor to explain
+                fallbackdata.append(
+                    SensorReading({'name': item['source'],
+                                       'states': ['Not Redundant'],
+                                       'state_ids': [3],
+                                       'health': pygconst.Health.Warning,
+                                       'type': 'Power'}, ''))
+            elif item['cmnid'] == 'FQXSFMA0041K':
+                fallbackdata.append(
+                    SensorReading({
+                        'name': 'Optane DCPDIMM',
+                        'health': pygconst.Health.Warning,
+                        'type': 'Memory',
+                        'states': [item['message']]},
+                        '')
+                )
+            else:
+                currevt = '{}:{}'.format(item['source'], item['message'])
+                if currevt in existingevts:
+                    continue
+                existingevts.add(currevt)
+                fallbackdata.append(SensorReading({
+                    'name': item['source'],
+                    'states': [item['message']],
+                    'health': itemseverity,
+                    'type': item['source'],
+                }, ''))
+        summary['badreadings'] = fallbackdata
+        return summary
+
     def get_description(self):
         description = self._do_web_request('/DeviceDescription.json')
         if description:
@@ -551,8 +646,12 @@ class OEMHandler(generic.OEMHandler):
 
     def _get_disk_firmware_single(self, diskent, prefix=''):
         bdata = {}
-        if not prefix and diskent.get('location', '').startswith('M.2'):
-            prefix = 'M.2-'
+        if not prefix:
+            location = diskent.get('location', '')
+            if location.startswith('M.2'):
+                prefix = 'M.2-'
+            elif location.startswith('7MM'):
+                prefix = '7MM-'
         diskname = 'Disk {1}{0}'.format(diskent['slotNo'], prefix)
         bdata['model'] = diskent[
             'productName'].rstrip()
@@ -629,12 +728,8 @@ class OEMHandler(generic.OEMHandler):
                             spares.append(diskinfo)
                         else:
                             disks.append(diskinfo)
-                    totalsize = pool['totalCapacityStr'].replace(
-                        'GB', '').replace('GiB', '')
-                    totalsize = int(float(totalsize) * 1024)
-                    freesize = pool['freeCapacityStr'].replace(
-                        'GB', '').replace('GiB', '')
-                    freesize = int(float(freesize) * 1024)
+                    totalsize = str_to_size(pool['totalCapacityStr'])
+                    freesize = str_to_size(pool['freeCapacityStr'])
                     pools.append(storage.Array(
                         disks=disks, raid=pool['rdlvlstr'], volumes=volumes,
                         id=(cid, pool['id']), hotspares=spares,
@@ -1121,6 +1216,29 @@ class OEMHandler(generic.OEMHandler):
         if usd['HttpPushUriTargetsBusy']:
             raise pygexc.TemporaryError('Cannot run multtiple updates to '
                                         'same target concurrently')
+        z = None
+        wrappedfilename = None
+        uxzcount = 0
+        needseek = False
+        if data and hasattr(data, 'read'):
+            if zipfile.is_zipfile(data):
+                needseek = True
+                z = zipfile.ZipFile(data)
+            else:
+                data.seek(0)
+        elif data is None and zipfile.is_zipfile(filename):
+            z = zipfile.ZipFile(filename)
+        if z:
+            for tmpname in z.namelist():
+                if tmpname.startswith('payloads/'):
+                    uxzcount += 1
+                    if tmpname.endswith('.uxz'):
+                        wrappedfilename = tmpname
+        if uxzcount == 1 and wrappedfilename:
+            filename = os.path.basename(wrappedfilename)
+            data = z.open(wrappedfilename)
+        elif needseek:
+            data.seek(0)
         upurl = usd['HttpPushUri']
         self._do_web_request(
             '/redfish/v1/UpdateService',
@@ -1160,10 +1278,15 @@ class OEMHandler(generic.OEMHandler):
             complete = False
             phase = "apply"
             statetype = 'TaskState'
-            while not complete:
+            # sometimes we get an empty pgress when transitioning from the apply phase to
+            # the validating phase; add a retry here so we don't exit the loop in this case
+            retry = 3
+            while not complete and retry > 0:
                 pgress = self._do_web_request(monitorurl, cache=False)
                 if not pgress:
-                    break
+                    retry -= 1
+                    time.sleep(3)
+                    continue
                 for msg in pgress.get('Messages', []):
                     if 'Verify failed' in msg.get('Message', ''):
                         raise Exception(msg['Message'])
@@ -1176,7 +1299,8 @@ class OEMHandler(generic.OEMHandler):
                 complete = state == 'Completed'
                 progress({'phase': phase, 'progress': pct})
                 if complete:
-                    if 'OperationTransitionedToJob' in pgress['Messages'][0]['MessageId']:
+                    msgs = pgress.get('Messages', [])
+                    if msgs and 'OperationTransitionedToJob' in msgs[0].get('MessageId', ''):
                         monitorurl = pgress['Messages'][0]['MessageArgs'][0]
                         phase = 'validating'
                         statetype = 'JobState'
@@ -1522,6 +1646,11 @@ class OEMHandler(generic.OEMHandler):
         yield "System"
         for key in natural_sort(hwmap):
             yield key
+        for cpuinv in self._get_cpu_inventory():
+            yield cpuinv[0]
+        for meminv in self._get_mem_inventory():
+            yield meminv[0]
+
 
     def get_inventory_of_component(self, compname):
         if compname.lower() == 'system':
@@ -1529,7 +1658,7 @@ class OEMHandler(generic.OEMHandler):
                 'UUID': self._varsysinfo.get('UUID', ''),
                 'Serial Number': self._varsysinfo.get('SerialNumber', ''),
                 'Manufacturer': self._varsysinfo.get('Manufacturer', ''),
-                'Product Name': self._varsysinfo.get('Model', ''),
+                'Product name': self._varsysinfo.get('Model', ''),
                 'Model': self._varsysinfo.get(
                     'SKU', self._varsysinfo.get('PartNumber', '')),
             }
@@ -1538,7 +1667,12 @@ class OEMHandler(generic.OEMHandler):
         try:
             return hwmap[compname]
         except KeyError:
-
+            for cpuinv in self._get_cpu_inventory():
+                if cpuinv[0] == compname:
+                    return cpuinv[1]
+            for meminv in self._get_mem_inventory():
+                if meminv[0] == compname:
+                    return meminv[1]
             return None
 
     def get_inventory(self, withids=False):
@@ -1546,7 +1680,7 @@ class OEMHandler(generic.OEMHandler):
             'UUID': self._varsysinfo.get('UUID', ''),
             'Serial Number': self._varsysinfo.get('SerialNumber', ''),
             'Manufacturer': self._varsysinfo.get('Manufacturer', ''),
-            'Product Name': self._varsysinfo.get('Model', ''),
+            'Product name': self._varsysinfo.get('Model', ''),
             'Model': self._varsysinfo.get(
                 'SKU', self._varsysinfo.get('PartNumber', '')),
         }
@@ -1614,7 +1748,7 @@ class OEMHandler(generic.OEMHandler):
                     if venid is not None:
                         bdata['PCI Vendor ID'] = '{0:04x}'.format(venid)
                     devid = fundata.get(self.ADP_DEVID, None)
-                    if devid is not None:
+                    if devid is not None and 'PCIE Device ID' not in bdata:
                         bdata['PCI Device ID'] = '{0:04x}'.format(devid)
                     venid = fundata.get(self.ADP_SUBVENID, None)
                     if venid is not None:
@@ -1714,8 +1848,12 @@ class OEMHandler(generic.OEMHandler):
 
     def get_disk_hardware(self, diskent, prefix=''):
         bdata = {}
-        if not prefix and diskent.get('location', '').startswith('M.2'):
-            prefix = 'M.2-'
+        if not prefix:
+            location = diskent.get('location', '')
+            if location.startswith('M.2'):
+                prefix = 'M.2-'
+            elif location.startswith('7MM'):
+                prefix = '7MM-'
         diskname = 'Disk {1}{0}'.format(diskent['slotNo'], prefix)
         bdata['Model'] = diskent['productName'].rstrip()
         bdata['Serial Number'] = diskent['serialNo'].rstrip()
